@@ -8,8 +8,6 @@
 #include "test.hh"
 
 #include <caf/actor.hpp>
-#include <caf/attach_stream_source.hpp>
-#include <caf/downstream.hpp>
 #include <caf/event_based_actor.hpp>
 #include <caf/exit_reason.hpp>
 #include <caf/send.hpp>
