@@ -345,6 +345,8 @@ template<typename T>
 ::zeek::ValPtr to_val(const hilti::rt::DeferredExpression<T>& t, ::zeek::TypePtr target, const std::string& location);
 template<typename T>
 ::zeek::ValPtr to_val(hilti::rt::integer::safe<T> i, ::zeek::TypePtr target, const std::string& location);
+template<typename T>
+::zeek::ValPtr to_val(const hilti::rt::ValueReference<T>& t, ::zeek::TypePtr target, const std::string& location);
 
 inline ::zeek::ValPtr to_val(const hilti::rt::Bool& b, ::zeek::TypePtr target, const std::string& location);
 inline ::zeek::ValPtr to_val(const hilti::rt::Address& d, ::zeek::TypePtr target, const std::string& location);
@@ -437,6 +439,14 @@ inline ::zeek::ValPtr to_val(hilti::rt::integer::safe<T> i, ::zeek::TypePtr targ
     }
 }
 
+template<typename T>
+::zeek::ValPtr to_val(const hilti::rt::ValueReference<T>& t, ::zeek::TypePtr target, const std::string& location) {
+    if ( auto* x = t.get() )
+        return to_val(*x, target, location);
+
+    return nullptr;
+}
+
 /**
  * Converts a Spicy-side signed bool to a Zeek value. The result is
  * returned with ref count +1.
@@ -712,7 +722,7 @@ inline ::zeek::ValPtr to_val(const T& t, ::zeek::TypePtr target, const std::stri
         // can't allow this ...
         throw InvalidValue("enum values with value max_int not supported by Zeek integration", location);
 
-    bro_int_t bt = (it >= 0 ? it : std::numeric_limits<::bro_int_t>::max());
+    zeek_int_t bt = (it >= 0 ? it : std::numeric_limits<::zeek_int_t>::max());
 
     return target->AsEnumType()->GetEnumVal(bt);
 }
