@@ -80,10 +80,7 @@ void BitTorrentTracker_Analyzer::DeliverStream(int len, const u_char* data, bool
 	{
 	analyzer::tcp::TCP_ApplicationAnalyzer::DeliverStream(len, data, orig);
 
-	assert(TCP());
-
-	if ( TCP()->IsPartial() )
-		// punt on partial.
+	if ( TCP() && TCP()->IsPartial() )
 		return;
 
 	if ( orig )
